@@ -4,7 +4,34 @@ void _store_enum (GIEnumInfo * info, gint value, GIArgument * arg);
 gint _retrieve_enum (GIEnumInfo * info, GIArgument * arg);
 
 static gpointer
-instance_sv_to_pointer (GICallableInfo *info, SV *sv)
+instance_sv_to_class_struct_pointer (SV *sv, GPerlI11nInvocationInfo *iinfo)
+{
+	gpointer pointer = NULL;
+	GType class_type = 0;
+	dwarn ("  -> gtype struct?\n");
+	if (gperl_sv_is_ref (sv)) { /* instance? */
+		const char *package = sv_reftype (SvRV (sv), TRUE);
+		class_type = gperl_type_from_package (package);
+	} else { /* package? */
+		class_type = gperl_type_from_package (SvPV_nolen (sv));
+	}
+	dwarn ("     class_type = %s (%lu), is_classed = %d\n",
+	       g_type_name (class_type), class_type, G_TYPE_IS_CLASSED (class_type));
+	if (G_TYPE_IS_CLASSED (class_type)) {
+		pointer = g_type_class_peek (class_type);
+		if (!pointer) {
+			/* If peek() produced NULL, the class has not been
+			 * instantiated yet and needs to be created. */
+			pointer = g_type_class_ref (class_type);
+			free_after_call (iinfo, (GFunc) g_type_class_unref, pointer);
+		}
+		dwarn ("     type class = %p\n", pointer);
+	}
+	return pointer;
+}
+
+static gpointer
+instance_sv_to_pointer (GICallableInfo *info, SV *sv, GPerlI11nInvocationInfo *iinfo)
 {
 	// We do *not* own container.
 	GIBaseInfo *container = g_base_info_get_container (info);
@@ -30,11 +57,16 @@ instance_sv_to_pointer (GICallableInfo *info, SV *sv)
 	    {
 		GType type = get_gtype ((GIRegisteredTypeInfo *) container);
 		if (!type || type == G_TYPE_NONE) {
-			dwarn ("  -> untyped record\n");
-			pointer = sv_to_struct (GI_TRANSFER_NOTHING,
-			                        container,
-			                        info_type,
-			                        sv);
+			if (g_struct_info_is_gtype_struct (container)) {
+				pointer = instance_sv_to_class_struct_pointer (sv, iinfo);
+			}
+			if (!pointer) {
+				dwarn ("  -> untyped record\n");
+				pointer = sv_to_struct (GI_TRANSFER_NOTHING,
+				                        container,
+				                        info_type,
+				                        sv);
+			}
 		} else {
 			dwarn ("  -> boxed: type=%s (%"G_GSIZE_FORMAT")\n",
 			       g_type_name (type), type);
@@ -126,21 +158,35 @@ sv_to_interface (GIArgInfo * arg_info,
 		if (may_be_null && !gperl_sv_is_defined (sv)) {
 			arg->v_pointer = NULL;
 		} else {
-			arg->v_pointer = gperl_get_object_check (sv, get_gtype (interface));
-		}
-		if (arg->v_pointer) {
-			GObject *object = arg->v_pointer;
-			if (transfer == GI_TRANSFER_NOTHING &&
-			    object->ref_count == 1 &&
-			    SvTEMP (sv) && SvREFCNT (SvRV (sv)) == 1)
+			/* GParamSpecs are represented as classes of
+			 * fundamental type, but gperl_get_object_check cannot
+			 * handle this.  So we do it here. */
+			if (info_type == GI_INFO_TYPE_OBJECT &&
+			    g_object_info_get_fundamental (interface))
 			{
-				cwarn ("*** Asked to hand out object without ownership transfer, "
-				       "but object is about to be destroyed; "
-				       "adding an additional reference for safety");
-				transfer = GI_TRANSFER_EVERYTHING;
-			}
-			if (transfer >= GI_TRANSFER_CONTAINER) {
-				g_object_ref (arg->v_pointer);
+				GType type = G_TYPE_FUNDAMENTAL (get_gtype (interface));
+				switch (type) {
+				    case G_TYPE_PARAM:
+					arg->v_pointer = SvGParamSpec (sv);
+					break;
+				    default:
+					ccroak ("sv_to_interface: Don't know how to handle fundamental type %s (%lu)\n",
+					        g_type_name (type), type);
+				}
+			} else {
+				arg->v_pointer = gperl_get_object_check (sv, get_gtype (interface));
+				if (arg->v_pointer && transfer == GI_TRANSFER_NOTHING &&
+				    ((GObject *) arg->v_pointer)->ref_count == 1 &&
+				    SvTEMP (sv) && SvREFCNT (SvRV (sv)) == 1)
+				{
+					cwarn ("*** Asked to hand out object without ownership transfer, "
+					       "but object is about to be destroyed; "
+					       "adding an additional reference for safety");
+					transfer = GI_TRANSFER_EVERYTHING;
+				}
+				if (transfer >= GI_TRANSFER_CONTAINER) {
+					g_object_ref (arg->v_pointer);
+				}
 			}
 		}
 		break;
@@ -310,6 +356,28 @@ interface_to_sv (GITypeInfo* info, GIArgument *arg, gboolean own, GPerlI11nInvoc
 
 	switch (info_type) {
 	    case GI_INFO_TYPE_OBJECT:
+		/* GParamSpecs are represented as classes of fundamental type,
+		 * but gperl_new_object cannot handle this.  So we do it
+		 * here. */
+		if (g_object_info_get_fundamental (interface)) {
+			GType type = G_TYPE_FUNDAMENTAL (get_gtype (interface));
+			switch (type) {
+			    case G_TYPE_PARAM:
+				sv = newSVGParamSpec (arg->v_pointer); /* does ref & sink */
+				/* FIXME: What if own=true and the pspec is not
+				 * floating?  Then we would leak.  We do not
+				 * have the API to detect this.  But it is
+				 * probably also quite rare. */
+				break;
+			    default:
+				ccroak ("interface_to_sv: Don't know how to handle fundamental type %s (%lu)\n",
+				        g_type_name (type), type);
+			}
+		} else {
+			sv = gperl_new_object (arg->v_pointer, own);
+		}
+		break;
+
 	    case GI_INFO_TYPE_INTERFACE:
 		sv = gperl_new_object (arg->v_pointer, own);
 		break;
