@@ -19,7 +19,7 @@ use strict;
 use warnings;
 use Glib;
 
-our $VERSION = '0.040';
+our $VERSION = '0.042';
 
 use Carp;
 $Carp::Internal{(__PACKAGE__)}++;
@@ -474,19 +474,7 @@ other widgets it's usually friendliest to avoid circularities in the first
 place, either by using weak references in the userdata, or possibly locating a
 parent dynamically with C<< $widget->get_ancestor >>.
 
-=head2 Miscellaneous
-
-In C you can only return one value from a function, and it is a common practice
-to modify pointers passed in to simulate returning multiple values.  In Perl,
-you can return lists; any functions which modify arguments are changed to
-return them instead.
-
-Arguments and return values that have the types GList or GSList or which are C
-arrays of values will be converted to and from references to normal Perl
-arrays.  The same holds for GHashTable and references to normal Perl hashes.
-
-You don't need to specify string lengths.  You can always use C<substr> to pass
-different parts of a string.
+=head2 Exception handling
 
 Anything that uses GError in C will C<croak> on failure, setting $@ to a
 magical exception object, which is overloaded to print as the
@@ -517,6 +505,28 @@ Glib::Error exception objects, you don't have to rely on string matching
 on a possibly localized error message; you can match errors by explicit and
 predictable conditions.  See L<Glib::Error> for more information.
 
+=head2 Output arguments, lists, hashes
+
+In C you can only return one value from a function, and it is a common practice
+to modify pointers passed in to simulate returning multiple values.  In Perl,
+you can return lists; any functions which modify arguments are changed to
+return them instead.
+
+Arguments and return values that have the types GList or GSList or which are C
+arrays of values will be converted to and from references to normal Perl
+arrays.  The same holds for GHashTable and references to normal Perl hashes.
+
+=head2 Object class functions
+
+Object class functions like C<Gtk3::WidgetClass::find_style_propery> can be
+called either with a package name or with an instance of the package.  For
+example:
+
+  Gtk3::WidgetClass::find_style_property ('Gtk3::Button', 'image-spacing')
+
+  my $button = Gtk3::Button->new;
+  Gtk3::WidgetClass::find_style_property ($button, 'image-spacing')
+
 =head1 DESCRIPTION FOR LIBRARY BINDING AUTHORS
 
 =head2 C<< Glib::Object::Introspection->setup >>
@@ -565,7 +575,7 @@ flattened so that they return plain lists.  For example
     'Gtk3::Window::list_toplevels'
   ]
 
-The function names refer to those after name corrections.  Functions occuring
+The function names refer to those after name corrections.  Functions occurring
 in C<flatten_array_ref_return_for> may also occur in C<class_static_methods>.
 
 =item handle_sentinel_boolean_for => [ function1, ... ]
@@ -582,7 +592,7 @@ be returned, and otherwise an empty list will be returned.
     'Gtk3::TreeSelection::get_selected'
   ]
 
-The function names refer to those after name corrections.  Functions occuring
+The function names refer to those after name corrections.  Functions occurring
 in C<handle_sentinel_boolean_for> may also occur in C<class_static_methods>.
 
 =item use_generic_signal_marshaller_for => [ [package1, signal1, [arg_converter1]], ... ]
@@ -663,12 +673,21 @@ wrapper for a function that expects a GValue, do this:
   ...
   my $type = ...; # somehow get the package name that
                   # corresponds to the correct GType
-  my $real_value =
+  my $wrapper =
     Glib::Object::Introspection::GValueWrapper->new ($type, $value);
   # now use Glib::Object::Introspection->invoke and
-  # substitute $real_value where you'd use $value
+  # substitute $wrapper where you'd use $value
   ...
 
+If you need to call a function that expects an already set-up GValue and
+modifies it, use C<get_value> on the wrapper afterwards to obtain the value.
+For example:
+
+  my $wrapper =
+    Glib::Object::Introspection::GValueWrapper->new ('Glib::Boolean', 0);
+  $box->child_get_property ($label, 'expand', $gvalue);
+  my $value = $gvalue->get_value
+
 =head2 Handling extendable enumerations
 
 If you need to handle extendable enumerations for which more than the
